

set -ex



test -f ${PREFIX}/lib/libamd${SHLIB_EXT}
test -f ${PREFIX}/lib/libamd.a
test -f ${PREFIX}/lib/libbtf${SHLIB_EXT}
test -f ${PREFIX}/lib/libbtf.a
test -f ${PREFIX}/lib/libcamd${SHLIB_EXT}
test -f ${PREFIX}/lib/libcamd.a
test -f ${PREFIX}/lib/libccolamd${SHLIB_EXT}
test -f ${PREFIX}/lib/libccolamd.a
test -f ${PREFIX}/lib/libcholmod${SHLIB_EXT}
test -f ${PREFIX}/lib/libcholmod.a
test -f ${PREFIX}/lib/libcolamd${SHLIB_EXT}
test -f ${PREFIX}/lib/libcolamd.a
test -f ${PREFIX}/lib/libcxsparse${SHLIB_EXT}
test -f ${PREFIX}/lib/libcxsparse.a
test -f ${PREFIX}/lib/libklu${SHLIB_EXT}
test -f ${PREFIX}/lib/libklu.a
test -f ${PREFIX}/lib/libldl${SHLIB_EXT}
test -f ${PREFIX}/lib/libldl.a
test -f ${PREFIX}/lib/librbio${SHLIB_EXT}
test -f ${PREFIX}/lib/librbio.a
test -f ${PREFIX}/lib/libspqr${SHLIB_EXT}
test -f ${PREFIX}/lib/libspqr.a
test -f ${PREFIX}/lib/libsuitesparseconfig${SHLIB_EXT}
test -f ${PREFIX}/lib/libsuitesparseconfig.a
test -f ${PREFIX}/lib/libumfpack${SHLIB_EXT}
test -f ${PREFIX}/lib/libumfpack.a
exit 0
