// =============================================================================
// === SuiteSparse_GPURuntime/Include/SuiteSparse_GPURuntime.hpp ===============
// =============================================================================

// SuiteSparse_GPURuntime, Copyright (c) 2013-2016, Timothy A Davis,
// Sencer Nuri Yeralan, and Sanjay Ranka.  All Rights Reserved.
// SPDX-License-Identifier: GPL-2.0+

//------------------------------------------------------------------------------

#ifndef SUITESPARSE_GPURUNTIME_HPP
#define SUITESPARSE_GPURUNTIME_HPP

// Version information:
#define SUITESPARSE_GPURUNTIME_DATE "Jan 17, 2023"
#define SUITESPARSE_GPURUNTIME_MAIN_VERSION   2
#define SUITESPARSE_GPURUNTIME_SUB_VERSION    0
#define SUITESPARSE_GPURUNTIME_SUBSUB_VERSION 3

#define SUITESPARSE_GPURUNTIME_VER_CODE(main,sub) ((main) * 1000 + (sub))
#define SUITESPARSE_GPURUNTIME_VERSION \
    SUITESPARSE_GPURUNTIME_VER_CODE(SUITESPARSE_GPURUNTIME_MAIN_VERSION,SUITESPARSE_GPURUNTIME_SUB_VERSION)

#endif
