// =============================================================================
// === GPUQREngine/Include/GPUQREngine.hpp =====================================
// =============================================================================

// GPUQREngine, Copyright (c) 2013, Timothy A Davis, Sencer Nuri Yeralan,
// and Sanjay Ranka.  All Rights Reserved.
// SPDX-License-Identifier: GPL-2.0+

// =============================================================================

#ifndef GPUQRENGINE_HPP
#define GPUQRENGINE_HPP

// Version information:
#define GPUQRENGINE_DATE "Jan 17, 2023"
#define GPUQRENGINE_MAIN_VERSION   2
#define GPUQRENGINE_SUB_VERSION    0
#define GPUQRENGINE_SUBSUB_VERSION 3

#define GPUQRENGINE_VER_CODE(main,sub) ((main) * 1000 + (sub))
#define GPUQRENGINE_VERSION \
    GPUQRENGINE_VER_CODE(GPUQRENGINE_MAIN_VERSION,GPUQRENGINE_SUB_VERSION)

#endif
